@@ -65,7 +65,7 @@
 # make it effectively proprietary.  To prevent this, the GPL assures that
 # patents cannot be used to render the program non-free.
 
-# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 2.5.2 - May 11, 2021
+# Open Generation and Transmission Operation and Expansion Planning Model with RES and ESS (openTEPES) - Version 3.1.2 - July 12, 2021
 # simplicity and transparency in power systems planning
 
 # Developed by
@@ -87,15 +87,16 @@
 import argparse
 import os
 # import pkg_resources
-from openTEPES.openTEPES import openTEPES_run
-import openTEPES
+from .openTEPES import openTEPES_run
+# from openTEPES.openTEPES import openTEPES_run
+# import openTEPES
 
 parser = argparse.ArgumentParser(description='Introducing main parameters...')
 parser.add_argument('--case',   type=str, default=None)
 parser.add_argument('--dir',    type=str, default=None)
 parser.add_argument('--solver', type=str, default=None)
 
-DIR    = os.path.dirname(openTEPES.__file__)
+DIR    = os.path.dirname(__file__)
 CASE   = '9n'
 SOLVER = 'glpk'
 
